
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

yarip.Loader = function()
{
	this.hasWhitelist = false;
	this.hasNoBlacklist = true;

	this.load = function(doc, map)
	{
		if (!doc || doc.nodeName != "#document" || !map) return false;

		var pageAddress = yarip.getAddress(doc, yarip.ADDRESS_MODE.page);
		if (!pageAddress) return false;

		var aAddress = yarip.getAddressArray(doc, map);
		if (!aAddress) return false;

		this.doWhitelisting(doc, pageAddress, aAddress, map);
		this.doBlacklisting(doc, pageAddress, aAddress, map);

		if (this.hasWhitelist && this.hasNoBlacklist) this.removeAllExceptWhitelisted(doc, pageAddress, map);

		var found = this.hasWhitelist || !this.hasNoBlacklist;
		this.hasWhitelist = false;
		this.hasNoBlacklist = true;
		return found;
	}

	this.doWhitelisting = function(doc, pageAddress, aAddress, map)
	{
		for (var i = aAddress.length - 1; i >= 0; i--)
		{
			var whitelist = map.obj[aAddress[i]].whitelist;

			for (var xValue in whitelist.xpaths)
			{
				var xpath = whitelist.xpaths[xValue];

				if (!xpath || !xpath.deep && !this.equalAddresses(pageAddress, aAddress[i])) continue;

				if (yarip.whitelist(doc, map, aAddress[i], xpath))
				{
					whitelist.xpaths[xValue].incrementFound();
					yarip.setChangeType(yarip.CHANGE_TYPE.minor);
					this.hasWhitelist = true;
				}
				else
				{
					whitelist.xpaths[xValue].incrementNotFound();
					yarip.setChangeType(yarip.CHANGE_TYPE.minor);
				}
			}
		}

		return true;
	}

	this.doBlacklisting = function(doc, pageAddress, aAddress, map)
	{
		for (var i = aAddress.length - 1; i >= 0; i--)
		{
			var blacklist = map.obj[aAddress[i]].blacklist;

			for (var xValue in blacklist.xpaths)
			{
				var xpath = blacklist.xpaths[xValue];

				if (!xpath || !xpath.deep && !this.equalAddresses(pageAddress, aAddress[i])) continue;

				if (yarip.blacklist(doc, map, aAddress[i], xpath))
				{
					blacklist.xpaths[xValue].incrementFound();
					yarip.setChangeType(yarip.CHANGE_TYPE.minor);
					this.hasNoBlacklist = false;
				}
				else
				{
					blacklist.xpaths[xValue].incrementNotFound();
					yarip.setChangeType(yarip.CHANGE_TYPE.minor);
				}
			}
		}
	}

	this.removeAllExceptWhitelisted = function(doc, pageAddress, map)
	{
		yarip.whitelist(doc, map, pageAddress, new yarip.XPath("/html/head/descendant-or-self::*[not(@status='blacklisted')]")); // whitelist the head and descendants (stylesheets)
		yarip.blacklist(doc, map, pageAddress, new yarip.XPath("//*[not(@status='whitelisted')]")); // remove all except whitelisted
	}

	this.equalAddresses = function(pageAddress, address)
	{
		pageAddress = pageAddress.replace(/[/]+$/, "");
		address = address.replace(/[/]+$/, "");
		if (address.match(yarip.noSchemeRegExp)) pageAddress = pageAddress.replace(/^[^:/?#]+:[/]{2}/, ""); // remove scheme
		return pageAddress == address;
	}
}
