
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

yarip.Overlay = function()
{
	this.enabled = null;
	this.active = false;
	this.noFlicker = null;

	this.enabledObserver = null;
	this.modeObserver = null;
	this.reloadObserver = null;
	this.flickerObserver = null;

	this.appcontent = null;
	this.contextMenu = null;
	this.panelContainer = null;
	this.stringbundle = null;
	this.flickerType = yarip.FLICKER_TYPE.auto;

	this.doc = null;
	this.control = null;
	this.loader = null;
	this.handler = null;
	this.highlighter = null;
	this.map = null;
	this.mode = null;

	this.start = function(event)
	{
		if (event.button != 0) return;

		var doc = content.document;

		if (this.active)
		{
			this.stop(doc, false);
			this.save(false);
			return;
		}

		if (!this.enabled)
		{
			this.toggleEnabled(true);
			return;
		}

		this.handler = null;
		if (!doc || !doc.defaultView.yaripHandler)
		{
			alert(this.stringbundle.getString("ERR_NO_HANDLER"));
			return;
		}
		this.handler = doc.defaultView.yaripHandler;

		if (!this.handler.start(doc, this.map, this.mode))
		{
			this.stop(doc, false);
			this.save(false);
			if (!this.map) alert(this.stringbundle.getString("ERR_NO_PAGES"));
			else alert(this.stringbundle.getString("ERR_HANDLER_DIDNT_START"));
			return;
		}

		this.active = true;
		this.doc = doc;
		this.setStatus(doc, "started");
	}

	this.stop = function(doc, noStateChange)
	{
		if (!this.active || !this.handler) return false;

		if (this.handler.stop())
		{
			yarip.setChangeType(yarip.CHANGE_TYPE.essential);
			this.save(false);
		}
		this.active = false;

		doc = doc ? doc : this.doc;
		if (!doc) return true;

		if (!noStateChange) this.setStatus(doc, "stopped");
		this.doc = null;

		return true;
	}

	this.save = function(reduce)
	{
		if (!this.map)
		{
			alert(this.stringbundle.getString("ERR_NO_PAGES"));
			return;
		}
		if (yarip.changeType == yarip.CHANGE_TYPE.none || yarip.changeType == yarip.CHANGE_TYPE.minor && !reduce) return;

		this.map.sortPages(reduce); // reduce=false/undefined; do not remove redundant xpaths (undo)
		this.control.savePages(this.map);
		yarip.setChangeType(reduce ? yarip.CHANGE_TYPE.none : yarip.CHANGE_TYPE.minor, true);

		this.reloadObserver.notify(yarip.changeType);
	}

	this.reload = function()
	{
		this.map = this.control.loadPages();
		yarip.undoObj = {}
		if (!this.map)
		{
			alert(this.stringbundle.getString("ERR_NO_PAGES"));
			return;
		}
	}

	this.whitelist = function(doc, node)
	{
		this.toggleEnabled(true);

		if (!doc || !node) return;

		var address = yarip.getAddress(doc, this.mode);
		if (!address) return;

		var xValue = yarip.calculateXPath(node);
		if (!xValue) return;
		xValue = prompt(this.stringbundle.getString("edit-for-whitelisting"), xValue + "/descendant-or-self::*");
		if (!xValue || xValue == "") return;

		if (yarip.whitelist(doc, this.map, address, new yarip.XPath(xValue, this.map.deep), true)) yarip.setChangeType(yarip.CHANGE_TYPE.essential);
	}

	this.blacklist = function(doc, node)
	{
		this.toggleEnabled(true);

		if (!doc || !node) return;

		var address = yarip.getAddress(doc, this.mode);
		if (!address) return;

		var xValue = yarip.calculateXPath(node);
		if (!xValue) return;
		xValue = prompt(this.stringbundle.getString("edit-for-blacklisting"), xValue);
		if (!xValue || xValue == "") return;

		if (yarip.blacklist(doc, this.map, address, new yarip.XPath(xValue, this.map.deep), true)) yarip.setChangeType(yarip.CHANGE_TYPE.essential);
	}

	this.removeTemporarily = function(doc, node)
	{
		this.toggleEnabled(true);

		if (!doc || !node) return;

		var address = yarip.getAddress(doc, this.mode);
		if (!address) return;

		var xValue = yarip.calculateXPath(node);
		if (!xValue) return;

		yarip.blacklist(doc, this.map, address, new yarip.XPath(xValue), false, true);
	}

	this.copyXPath = function(node)
	{
		if (!node) return;

		var xValue = yarip.calculateXPath(node);
		if (!xValue) return;

		var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
		clipboardHelper.copyString(xValue);
	}

	this.startHighlighting = function(node)
	{
		if (!node) return;

		this.highlighter.doc = node.ownerDocument;
		this.highlighter.element = node;
		this.highlighter.select();
	}

	this.stopHighlighting = function()
	{
		this.highlighter.doc = null;
		this.highlighter.reset();
	}

	this.managePages = function()
	{
		this.stop();
		this.save(true);
		yarip.undoObj = {}
		var aAddress = yarip.getAddressArray(content.document, this.map);
		var pageAddress = aAddress && aAddress[0] ? aAddress[0] : null;
		window.openDialog("chrome://yarip/content/pagedialog.xul", "pagedialog", "chrome,resizable", this.control, this.map, pageAddress);
	}

	this.setMode = function(mode, clear, dontNotify)
	{
		if (typeof mode != "number") return;

		this.stop();
		this.save(false);
		this.toggleDeep(true);

		if (clear)
		{
			yarip.$("yarip-page-menuitem").setAttribute("checked", "false");
			yarip.$("yarip-fqdn-menuitem").setAttribute("checked", "false");
			yarip.$("yarip-sld-menuitem").setAttribute("checked", "false");
			yarip.$("yarip-user-menuitem").setAttribute("checked", "false");
		}

		switch (mode)
		{
			case yarip.ADDRESS_MODE.page:
				yarip.$("yarip-page-menuitem").setAttribute("checked", "true");
				break;

			case yarip.ADDRESS_MODE.fqdn:
				yarip.$("yarip-fqdn-menuitem").setAttribute("checked", "true");
				break;

			case yarip.ADDRESS_MODE.sld:
				yarip.$("yarip-sld-menuitem").setAttribute("checked", "true");
				break;

			case yarip.ADDRESS_MODE.user:
				var doc = content.document;
				var address = yarip.getAddress(doc, this.mode);
				if (!address)
				{
					yarip.address = null;
					if (this.mode != yarip.ADDRESS_MODE.user) this.setMode(this.mode, true);
					return;
				}
				address = prompt("Edit address", address);
				if (!address || address == "")
				{
					yarip.address = null;
					if (this.mode != yarip.ADDRESS_MODE.user) this.setMode(this.mode, true);
					return;
				}
				yarip.address = address;
				yarip.$("yarip-user-menuitem").setAttribute("checked", "true");
		}

		this.mode = mode;
		if (!dontNotify) this.modeObserver.notify(mode);
	}

	this.updateMode = function()
	{
		var mode = this.control.getValue(yarip.MODE, yarip.ADDRESS_MODE.fqdn, yarip.INTEGER);
		if (mode != yarip.ADDRESS_MODE.user) this.setMode(mode, true, true);
	}

	this.toggleEnabled = function(enabled, dontNotify)
	{
		if (enabled == this.enabled) return;

		if (enabled != true && enabled != false) enabled = !this.enabled;
		var notify = !dontNotify /*&& this.enabled != enabled*/;
		this.enabled = enabled;

		if (enabled)
		{
			this.flickerType = this.control.getValue(yarip.FLICKER, yarip.FLICKER_TYPE.auto, yarip.INTEGER);
			this.toggleNoFlicker(this.flickerType == yarip.FLICKER_TYPE.prevent ? true : false);
		}
		else
		{
			this.stop(null, true);
			this.save(true);

			this.flickerType = yarip.FLICKER_TYPE.allow;
			this.toggleNoFlicker(false);
		}

		if (notify) this.enabledObserver.notify(enabled);

		yarip.$("yarip-enabled-menuitem").setAttribute("checked", "" + enabled);
		this.setStatus(content.document, enabled ? "enabled" : "disabled");
	}

	this.updateEnabled = function()
	{
		var enabled = this.control.getValue(yarip.ENABLED, true, yarip.BOOLEAN);
		this.toggleEnabled(enabled, true);
	}

	this.setFlickerType = function(type, clear, dontNotify)
	{
		if (typeof type != "number") return;

		if (clear)
		{
			yarip.$("yarip-allow-menuitem").setAttribute("checked", "false");
			yarip.$("yarip-prevent-menuitem").setAttribute("checked", "false");
			yarip.$("yarip-auto-menuitem").setAttribute("checked", "false");
		}

		switch (type)
		{
			case yarip.FLICKER_TYPE.allow:
				yarip.$("yarip-allow-menuitem").setAttribute("checked", "true");
				this.flickerType = type;
				this.toggleNoFlicker(false);
				break;

			case yarip.FLICKER_TYPE.prevent:
				yarip.$("yarip-prevent-menuitem").setAttribute("checked", "true");
				this.flickerType = type;
				this.toggleNoFlicker(true);
				break;

			case yarip.FLICKER_TYPE.auto:
				yarip.$("yarip-auto-menuitem").setAttribute("checked", "true");
				this.flickerType = type;
				break;

			default:
				this.flickerType = yarip.FLICKER_TYPE.auto;
		}

		if (!dontNotify) this.flickerObserver.notify(this.flickerType);
	}

	this.updateFlickerType = function()
	{
		var type = this.control.getValue(yarip.FLICKER, yarip.FLICKER_TYPE.auto, yarip.INTEGER);
		this.setFlickerType(type, true, true);
	}

	this.toggleNoFlicker = function(noFlicker)
	{
		switch (this.flickerType)
		{
			case yarip.FLICKER_TYPE.allow:
				this.control.setValue("nglayout.initialpaint.delay", null, yarip.RESET);
				noFlicker = false;
				break;

			case yarip.FLICKER_TYPE.prevent:
				this.control.setValue("nglayout.initialpaint.delay", 5000, yarip.INTEGER);
				noFlicker = true;
				break;

			case yarip.FLICKER_TYPE.auto:
				this.control.setValue("nglayout.initialpaint.delay", 5000, yarip.INTEGER);
		}

		if (noFlicker == this.noFlicker) return;
		this.noFlicker = noFlicker && this.enabled;

		this.injectCSS("chrome://yarip/content/noflicker.css", this.noFlicker)
	}

	this.toggleDeep = function(deep)
	{
		if (deep == this.map.deep) return;

		if (deep != true && deep != false) deep = !this.map.deep;
		this.map.deep = deep;

		yarip.$("yarip-deep-menuitem").setAttribute("checked", "" + this.map.deep);
	}

	this.checkFirstRun = function()
	{
		if (!this.control) return;

		// FIRST RUN
		var firstrun = this.control.getValue(yarip.FIRSTRUN, true, yarip.BOOLEAN);
		if (firstrun)
		{
			this.control.setValue(yarip.FIRSTRUN, false, yarip.BOOLEAN);
		}

		// UPGRADE
		var version = this.control.getValue(yarip.VERSION, "", yarip.STRING);
		var extensionManager = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
		var current = extensionManager.getItemForID("yarip@mozdev.org").version;
		if (version != current && !firstrun)
		{
			this.control.setValue(yarip.VERSION, current, yarip.STRING);

			if (!current.match(/^0\.1\.[4-6]$/)) this.control.setValue("extensions.yarip.noFlicker.value", null, yarip.RESET); // renamed to flickerType
		}
	}

	this.handleEvent = function(event)
	{
		if (!window || !("yarip" in window)) return;

		switch(event.type)
		{
			case "load":
				this.control = new yarip.Control();
				this.map = this.control.loadPages();
				this.loader = new yarip.Loader();
				this.highlighter = new yarip.Handler();

				this.enabledObserver = new yarip.Observer(
					yarip.ENABLED_ROOT, // preference root
					yarip.VALUE, // preference
					function() { yarip.overlay.updateEnabled(); }, // call-back function
					function(enabled) { yarip.overlay.control.setValue(yarip.ENABLED, enabled, yarip.BOOLEAN); } // notify function
				);
				this.modeObserver = new yarip.Observer(
					yarip.MODE_ROOT,
					yarip.VALUE,
					function() { yarip.overlay.updateMode(); },
					function(mode) { yarip.overlay.control.setValue(yarip.MODE, mode, yarip.INTEGER); }
				);
				this.reloadObserver = new yarip.Observer(
					yarip.RELOAD_ROOT,
					yarip.VALUE,
					function() {
						var type = yarip.overlay.control.getValue(yarip.CHANGE, yarip.CHANGE_TYPE.none, yarip.INTEGER);
						yarip.setChangeType(type);
						yarip.overlay.reload();
					},
					function(type) {
						yarip.overlay.control.setValue(yarip.CHANGE, type, yarip.INTEGER);
						yarip.overlay.control.setValue(yarip.RELOAD, (new Date()).getTime(), yarip.STRING);
					}
				);
				this.flickerObserver = new yarip.Observer(
					yarip.FLICKER_ROOT,
					yarip.VALUE,
					function() { yarip.overlay.updateFlickerType(); },
					function(type) { yarip.overlay.control.setValue(yarip.FLICKER, type, yarip.INTEGER); }
				);
				this.alwaysUseIndexObserver = new yarip.Observer(
					yarip.INDEX_ROOT,
					yarip.VALUE,
					function() { yarip.alwaysUseIndex = yarip.overlay.control.getValue(yarip.INDEX, true, yarip.BOOLEAN); }
//					,function(alwaysUseIndex) { yarip.overlay.control.setValue(yarip.INDEX, alwaysUseIndex, yarip.BOOLEAN); }
				);

				this.appcontent = yarip.$("appcontent");
				this.contextMenu = yarip.$("contentAreaContextMenu");
				if (gBrowser) this.panelContainer = gBrowser.mPanelContainer;
				this.stringbundle = yarip.$("yarip-overlay-stringbundle");
				yarip.alwaysUseIndex = this.control.getValue(yarip.INDEX, true, yarip.BOOLEAN);
				yarip.setChangeType(this.control.getValue(yarip.CHANGE, yarip.CHANGE_TYPE.none, yarip.INTEGER));

				this.checkFirstRun();
				this.setMode(yarip.ADDRESS_MODE.fqdn, true);
				this.updateFlickerType();
				this.updateEnabled();
				this.injectCSS("chrome://yarip/content/blacklist.css", true);

				if (this.appcontent) this.appcontent.addEventListener("DOMContentLoaded", this, false);
				if (this.contextMenu) this.contextMenu.addEventListener("popupshowing", this, false);
				if (this.panelContainer) this.panelContainer.addEventListener("select", this, false);
				window.removeEventListener("load", this, false);

				break;

			case "unload":
				this.save(true);

				this.enabledObserver.unregister();
				this.modeObserver.unregister();
				this.reloadObserver.unregister();
				this.flickerObserver.unregister();
				this.alwaysUseIndexObserver.unregister();

				if (this.appcontent) this.appcontent.removeEventListener("DOMContentLoaded", this, false);
				if (this.contextMenu) this.contextMenu.removeEventListener("popupshowing", this, false);
				if (this.panelContainer) this.panelContainer.removeEventListener("select", this, false);
				window.removeEventListener("load", this, false);
				window.removeEventListener("unload", this, false);
				window.removeEventListener("DOMMenuItemActive", this, false);
				window.removeEventListener("keypress", this, false);

				break;

			case "DOMContentLoaded":
				this.stop();
				this.toggleDeep(true);

				var doc = event.originalTarget;
				if (!doc) return;

				var inFrame = doc.defaultView.frameElement != null;
				yarip.undoObj = {}
				var rootDoc = doc;

				// HANDLER
				if (!inFrame)
				{
					doc.defaultView.yaripHandler = new yarip.Handler(null, function() { yarip.overlay.stop(); });
					if (this.enabled) this.setStatus(doc, "stopped");
				}
				else
				{
					while (rootDoc.defaultView.frameElement) rootDoc = rootDoc.defaultView.frameElement.ownerDocument; // get root doc
					if (rootDoc.defaultView.yaripHandler) rootDoc.defaultView.yaripHandler.addHandler(new yarip.Handler(doc, function() { yarip.overlay.stop(); }));
				}

				// LOADER
				if (doc.nodeName == "#document" && this.enabled && this.map)
				{
					if (this.loader.load(doc, this.map))
					{
						this.toggleNoFlicker(true);
						this.setStatus(rootDoc, "found");
					}
					else if (!inFrame) this.toggleNoFlicker(false);

					if (this.mode == yarip.ADDRESS_MODE.user) this.setMode(yarip.ADDRESS_MODE.fqdn, true);
				}
				else this.toggleNoFlicker(false);

				if (doc.body) doc.body.setAttribute("status", "show");

				break;

			case "DOMMenuItemActive":
				this.toggleNoFlicker(false);
				break;

			case "keypress":
				if (event.ctrlKey) this.toggleNoFlicker(false);
				break;

			case "popupshowing":
				this.stop();
//				this.save(false);

				var node = document.popupNode;
				var doc = node.ownerDocument;

				// YARIP-MENU 1/2
				var hasMenus = false;
				var hidden = !yarip.getAddress(doc, yarip.overlay.mode);

				if (!hidden)
				{
					// ELEMENT-MENU
					var tmpHidden = hidden || !node.localName || node.localName.search(/^(html|body)$/i) > -1;
					if (!tmpHidden) yarip.$("yarip-element-menu").setAttribute("label", this.stringbundle.getFormattedString("menuitem-element", [node.localName.toUpperCase()]));
					yarip.$("yarip-element-menu").hidden = tmpHidden;
					hasMenus = hasMenus || !tmpHidden;

					// PARENT-MENU
					tmpHidden = hidden || !node.parentNode || !node.parentNode.localName || node.parentNode.localName.search(/^(html|body)$/i) > -1;
					if (!tmpHidden) yarip.$("yarip-parent-menu").setAttribute("label", this.stringbundle.getFormattedString("menuitem-parent", [node.parentNode.localName.toUpperCase()]));
					yarip.$("yarip-parent-menu").hidden = tmpHidden;
					hasMenus = hasMenus || !tmpHidden;

					// FRAME-MENU
					tmpHidden = hidden || !gContextMenu.inFrame;
					var frameMenu = yarip.$("yarip-frame-menu");
					if (!tmpHidden)
					{
						var frame = node.ownerDocument.defaultView.frameElement;
						frameMenu.setAttribute("label", frame.localName.toUpperCase());
						if (!yarip.getAddress(frame.ownerDocument, yarip.overlay.mode)) frameMenu.setAttribute("disabled", true);
						else frameMenu.removeAttribute("disabled");
					}
					frameMenu.hidden = tmpHidden;
					hasMenus = hasMenus || !tmpHidden;

					// UNDO-MENU
					var undoMenu = yarip.$("yarip-undo-menupopup");
					var menuitem = undoMenu.firstChild;
					while (menuitem && menuitem.localName != "menuseparator")
					{
						var next = menuitem.nextSibling;
						undoMenu.removeChild(menuitem);
						menuitem = next;
					}
					var found = false;
					for (var xValue in yarip.undoObj)
					{
						var obj = yarip.undoObj[xValue];
						if (!obj || obj.document != doc) continue;

						var menuitem = document.createElement("menuitem");
						menuitem.setAttribute("label", this.stringbundle.getFormattedString("undo-" + obj.type, [xValue]));
						menuitem.obj = obj;
						menuitem.setAttribute("oncommand", "yarip.overlay.undo(this.obj)");
						undoMenu.insertBefore(menuitem, undoMenu.firstChild);
						found = true;
					}
					yarip.$("yarip-undo-menu-sep").hidden = !hasMenus || !found;
					yarip.$("yarip-undo-menu").hidden = !found;
					hasMenus = hasMenus || found;
				}

				// YARIP-MENU 2/2
				yarip.$("yarip-menu").hidden = hidden || !hasMenus;
				yarip.$("yarip-menu-sep").hidden = hidden || !hasMenus;

				break;

			case "select":
				if (this.mode == yarip.ADDRESS_MODE.user) this.setMode(yarip.ADDRESS_MODE.fqdn, true);
				this.setStatus(gBrowser.getBrowserAtIndex(gBrowser.mTabContainer.selectedIndex).contentDocument);
		}
	}

	this.undoAll = function(doc)
	{
		if (!doc) return;

		for (var xValue in yarip.undoObj)
		{
			var obj = yarip.undoObj[xValue];
			if (obj && obj.document == doc) this.undo(obj);
		}
	}

	this.undo = function(obj)
	{
		if (!obj) return;

		yarip.resetOnAddress(obj.document, this.map, obj.address, new yarip.XPath(obj.xValue), true);
		yarip.undoObj[obj.xValue] = null;
	}

	this.injectCSS = function(path, value)
	{
		if (!path) return;

		var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var uri = ios.newURI(path, yarip.CHARSET, null); // uri, charset, baseuri
		var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"].getService(Components.interfaces.nsIStyleSheetService);
		if (value) { if (!sss.sheetRegistered(uri, sss.USER_SHEET)) sss.loadAndRegisterSheet(uri, sss.USER_SHEET); }
		else { if (sss.sheetRegistered(uri, sss.USER_SHEET)) sss.unregisterSheet(uri, sss.USER_SHEET); }
	}

	this.setStatus = function(doc, status)
	{
		if (!doc) return;

		if (this.enabled)
		{
			if (!status)
			{
				status = doc.defaultView.yaripStatus;
				switch (status)
				{
					case "enabled":
					case "stopped":
					case "found":
						break;

					case "disabled":
					case "started": if (this.active) break;
					default:
						status = "enabled";
				}
			}
		}
		else status = "disabled";

		if (doc.defaultView) doc.defaultView.yaripStatus = status;
		if (doc == gBrowser.getBrowserAtIndex(gBrowser.mTabContainer.selectedIndex).contentDocument)
		{
			yarip.$("yarip-statusbarpanel").setAttribute("status", status);
			yarip.$("yarip-statusbarpanel").setAttribute("tooltiptext", this.stringbundle.getString("status-" + status));
		}
	}
}
