
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

yarip.Page = function(name, whitelist, blacklist)
{
	this.name = "";
	this.whitelist = whitelist ? whitelist : new yarip.List("whitelist");
	this.blacklist = blacklist ? blacklist : new yarip.List("blacklist");

	this.setName = function(pName)
	{
		if (pName) this.name = "" + pName;
	}

	this.getName = function()
	{
		return this.name;
	}

	this.reset = function()
	{
		this.whitelist.reset();
		this.blacklist.reset();
	}

	this.clone = function(purge)
	{
		return new this.constructor(this.name, this.whitelist.clone(purge), this.blacklist.clone(purge));
	}

	this.generateXml = function()
	{
		var tmp = this.whitelist.generateXml() + this.blacklist.generateXml();
		if (tmp != "") return "\t<page name=\"" + this.name + "\">\n" + tmp + "\t</page>\n";
		else return "";
	}

	this.setName(name);
}

yarip.PageSorter = function(a, b)
{
	var aArr = a.name.match(yarip.domainRegExp)[0].split('.').reverse();
	var bArr = b.name.match(yarip.domainRegExp)[0].split('.').reverse();
	for (var i = 1; i >= 0; i--)
	{
		if (aArr[i] < bArr[i]) return -1;
		if (aArr[i] > bArr[i]) return 1;
	}
	for (var i = 2; i < aArr.length && i < bArr.length; i++)
	{
		if (aArr[i] < bArr[i]) return -1;
		if (aArr[i] > bArr[i]) return 1;
	}
	if (aArr.length < bArr.length) return -1;
	if (aArr.length > bArr.length) return 1;
	var aIsDomain = a.name.search(/^[^:/?&#]+$/) > -1;
	var bIsDomain = b.name.search(/^[^:/?&#]+$/) > -1;
	if (aIsDomain && bIsDomain) return 0;
	if (aIsDomain && !bIsDomain) return -1;
	if (!aIsDomain && bIsDomain) return 1;
	return yarip.normalSort(a.name, b.name);
}
