
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

yarip.PageDialog = function()
{
	this.control = null;
	this.map = null;
	this.stringbundle = null;
	this.reloadObserver = null;

	this.tabSelect = function(tab)
	{
		if (!tab) return;

		this.tab = tab;
	}

	this.setXPathValue = function(value)
	{
		if (!value) return;
		if (!this.tabs[this.tab].xpath) return;

		this.tabs[this.tab].xpath.setValue(value);
	}

	this.setDeep = function(deep)
	{
		if (deep != true && deep != false) return;
		if (!this.tabs[this.tab].xpath) return;

		this.tabs[this.tab].xpath.setDeep(deep);
	}

	this.checkXPath = function(xValue)
	{
		if (!xValue) return false;

		try { document.evaluate(xValue, document.createElement("div"), null, XPathResult.ANY_TYPE, null); }
		catch (e)
		{
//			alert(e.message);
			alert(this.stringbundle.getString("ERR_INVALID_XPATH"));
			return false;
		}

		return true;
	}

	this.addXPath = function()
	{
		if (this.listboxPages.selectedIndex < 0) return;
		if (!this.tabs[this.tab].xpath) return;

		var xValue = this.tabs[this.tab].xpath.getValue();
		if (xValue != "" && this.checkXPath(xValue))
		{
			this.tabs[this.tab].list.add(new yarip.XPath(this.tabs[this.tab].xpath.getValue()));
			this.tabs[this.tab].list.sortXPaths(true);
			this.refreshXPaths(true);
			yarip.setChangeType(yarip.CHANGE_TYPE.essential);
		}
		else this.tabs[this.tab].textbox.select();
	}

	this.updateXPath = function()
	{
		if (!this.tabs[this.tab].xpath || this.tabs[this.tab].tree.currentIndex < 0) return;

		var xValue = this.tabs[this.tab].xpath.getValue();
		if (xValue != "" && this.checkXPath(xValue))
		{
			this.tabs[this.tab].list.remove(new yarip.XPath(this.tabs[this.tab].aXPaths[this.tabs[this.tab].tree.currentIndex][0]));
			this.tabs[this.tab].list.add(this.tabs[this.tab].xpath);
			this.tabs[this.tab].list.sortXPaths(true);
			this.refreshXPaths(true);
			yarip.setChangeType(yarip.CHANGE_TYPE.essential);
		}
		else this.tabs[this.tab].textbox.select();
	}

	this.removeXPath = function()
	{
		if (!this.tabs[this.tab].xpath || this.tabs[this.tab].tree.currentIndex < 0) return;

		this.tabs[this.tab].list.remove(this.tabs[this.tab].xpath);
		this.refreshXPaths(true);
		yarip.setChangeType(yarip.CHANGE_TYPE.essential);
	}

	this.removeAllXPaths = function()
	{
		if (!this.tabs[this.tab].list || this.tabs[this.tab].list.length == 0) return;

		this.tabs[this.tab].list.reset();
		this.refreshXPaths(true);
		yarip.setChangeType(yarip.CHANGE_TYPE.essential);
	}

	this.newPage = function()
	{
		if (!this.map || !window || !("yarip" in window)) return;

		this.renamePage(new yarip.Page("www.example.net"));
	}

	this.copyPage = function()
	{
		if (!this.map || this.listboxPages.selectedIndex < 0) return;

		var page = this.listboxPages.getItemAtIndex(this.listboxPages.selectedIndex).page.clone(true);
		page.setName("www.example.net");
		this.renamePage(page);
	}

	this.checkPage = function(name)
	{
		if (!name) return false;

		try { if (!name.match(yarip.uriRegExp)) throw new yarip.Exception(this.stringbundle.getString("ERR_INVALID_PAGE")); }
		catch (e)
		{
			alert(e.message);
			return false;
		}

		return true;
	}

/*	this.selectTab = function(listboxItem)
	{
		if (!listboxItem) return;

		if (listboxItem.page.whitelist.length == 0 && listboxItem.page.blacklist.length > 0) yarip.$("tabbox").selectedIndex = 1;
	}*/

	this.selectListboxItem = function(pageName)
	{
		if (!pageName) return null;

		for (var i = 0; i < this.listboxPages.getRowCount(); i++)
		{
			var item = this.listboxPages.getItemAtIndex(i);
			if (item.page.getName() == pageName)
			{
				this.listboxPages.selectedIndex = i;
				this.listboxPages.ensureIndexIsVisible(i);
				return item;
			}
		}

		return null;
	}

	this.renamePage = function(page)
	{
		if (!page)
		{
			if (this.listboxPages.selectedIndex < 0) return false;
			page = this.listboxPages.getItemAtIndex(this.listboxPages.selectedIndex).page;
		}
		var name = prompt("Enter name of page:", page.getName());
		if (!name || name == "" || !this.checkPage(name)) return false;
//		else this.map.obj[page.getName()] = null;

		page.setName(name);
		if (!this.map.obj[page.getName()]) this.map.obj[page.getName()] = page;
		else
		{
			var tmp = this.map.obj[page.getName()];
			for (var v in page.whitelist.xpaths) tmp.whitelist.add(page.whitelist.xpaths[v]); // append/overwrite
			for (var v in page.blacklist.xpaths) tmp.blacklist.add(page.blacklist.xpaths[v]);
		}
		this.map.sortPages(true);
		yarip.setChangeType(yarip.CHANGE_TYPE.essential);

		this.refreshPages(true);
		this.selectListboxItem(page.getName());

		return true;
	}

	this.removePage = function()
	{
		if (this.listboxPages.selectedIndex < 0) return;

		this.map.obj[this.listboxPages.removeItemAt(this.listboxPages.selectedIndex).page.getName()] = null;
		this.refreshPages();
		yarip.setChangeType(yarip.CHANGE_TYPE.essential);
	}

	this.refreshPages = function(load)
	{
		if (!window || !("yarip" in window)) return;

		if (load)
		{
			this.listboxPages.removeEventListener("select", this, false);
			while (this.listboxPages.getRowCount() > 0 && this.listboxPages.removeItemAt(0));
			this.listboxPages.addEventListener("select", this, false);

			for (var pageAddress in this.map.obj)
			{
				var page = this.map.obj[pageAddress];
				if (!page) continue;
				var listitem = this.listboxPages.appendItem(pageAddress);
				listitem.page = page;
			}
		}

		var page = null;
		if (load || this.listboxPages.selectedIndex < 0) page = new yarip.Page();
		else page = this.listboxPages.getSelectedItem(0).page;

		this.dialogheaderPage.setAttribute("title", page.getName());
		this.tabs["whitelist"].list = page.whitelist;
		this.tabs["blacklist"].list = page.blacklist;

		this.refreshXPaths(true);
	}

	this.refreshXPaths = function(load)
	{
		if (!window || !("yarip" in window)) return;

		if (load)
		{
			this.tabs["whitelist"].aXPaths = this.tabs["whitelist"].list ? this.tabs["whitelist"].list.getXPathsArray() : [];
			this.tabs["whitelist"].tree.view = new yarip.TreeView(this.tabs["whitelist"].aXPaths, this.tabs["whitelist"].columns, this.tabs["whitelist"].aXPaths.length);

			this.tabs["blacklist"].aXPaths = this.tabs["blacklist"].list ? this.tabs["blacklist"].list.getXPathsArray() : [];
			this.tabs["blacklist"].tree.view = new yarip.TreeView(this.tabs["blacklist"].aXPaths, this.tabs["blacklist"].columns, this.tabs["blacklist"].aXPaths.length);
		}

		if (load || this.tabs["whitelist"].tree.currentIndex < 0)
		{
			this.tabs["whitelist"].xpath = new yarip.XPath();
			this.tabs["whitelist"].update.hidden = true;
		}
		else
		{
			var xValue = this.tabs["whitelist"].aXPaths[this.tabs["whitelist"].tree.currentIndex][0];
			if (this.tabs["whitelist"].list.xpaths[xValue]) this.tabs["whitelist"].xpath = this.tabs["whitelist"].list.xpaths[xValue].clone(new yarip.XPath());
			this.tabs["whitelist"].update.hidden = false;
		}

		if (load || this.tabs["blacklist"].tree.currentIndex < 0)
		{
			this.tabs["blacklist"].xpath = new yarip.XPath();
			this.tabs["blacklist"].update.hidden = true;
		}
		else
		{
			var xValue = this.tabs["blacklist"].aXPaths[this.tabs["blacklist"].tree.currentIndex][0];
			if (this.tabs["blacklist"].list.xpaths[xValue]) this.tabs["blacklist"].xpath = this.tabs["blacklist"].list.xpaths[xValue].clone(new yarip.XPath());
			this.tabs["blacklist"].update.hidden = false;
		}

		this.tabs["whitelist"].textbox.value = this.tabs["whitelist"].xpath.getValue();
		this.tabs["whitelist"].menulist.selectedIndex = this.tabs["whitelist"].xpath.getDeep() ? 0 : 1;

		this.tabs["blacklist"].textbox.value = this.tabs["blacklist"].xpath.getValue();
		this.tabs["blacklist"].menulist.selectedIndex = this.tabs["blacklist"].xpath.getDeep() ? 0 : 1;
	}

	this.save = function()
	{
		if (!this.map)
		{
			alert(this.stringbundle.getString("ERR_NO_PAGES"));
			return;
		}
		if (yarip.changeType == yarip.CHANGE_TYPE.none) return;

		this.map.sortPages(true);
		this.control.savePages(this.map);
		yarip.setChangeType(yarip.CHANGE_TYPE.none, true);

		this.reloadObserver.notify(yarip.changeType);
	}

	this.reload = function()
	{
		var pageName = null;
		if (this.listboxPages.selectedIndex > 0) pageName = this.listboxPages.getSelectedItem(0).page.getName();

		this.map = this.control.loadPages();
		if (!this.map)
		{
			alert(this.stringbundle.getString("ERR_NO_PAGES"));
			return;
		}

		this.refreshPages(true);
		if (pageName) this.selectListboxItem(pageName);
//		if (pageName) this.selectTab(this.selectListboxItem(pageName));
	}

	this.handleEvent = function(event)
	{
		if (!event) return;

		switch(event.target.id)
		{
			case "listbox-pages":
				this.refreshPages();
				break;

			case "whitelist-tree":
			case "blacklist-tree":
				this.refreshXPaths();
		}
	}

	this.load = function()
	{
		if (!window || !("yarip" in window)) return;

		var pageName = null;
		if ("arguments" in window && window.arguments.length > 0)
		{
			this.control = window.arguments[0];
			this.map = window.arguments[1];
			pageName = window.arguments[2];
		}

		this.listboxPages = yarip.$("listbox-pages");
		this.dialogheaderPage = yarip.$("dialogheader-page");
		this.stringbundle = yarip.$("page-dialog-stringbundle");

		this.tab = "whitelist";
		this.tabs = {};

		this.tabs["whitelist"] = {};
		this.tabs["whitelist"].textbox = yarip.$("whitelist-xpath");
		this.tabs["whitelist"].menulist = yarip.$("whitelist-deep");
		this.tabs["whitelist"].add = yarip.$("whitelist-add-xpath");
		this.tabs["whitelist"].update = yarip.$("whitelist-update-xpath");
		this.tabs["whitelist"].tree = yarip.$("whitelist-tree");
		this.tabs["whitelist"].columns = {
			wlXPath: 0,
			wlDeep: 1,
			wlFound: 2,
			wlNotFound: 3,
			wlWhitelisted: 4
		}

		this.tabs["blacklist"] = {};
		this.tabs["blacklist"].textbox = yarip.$("blacklist-xpath");
		this.tabs["blacklist"].menulist = yarip.$("blacklist-deep");
		this.tabs["blacklist"].add = yarip.$("blacklist-add-xpath");
		this.tabs["blacklist"].update = yarip.$("blacklist-update-xpath");
		this.tabs["blacklist"].tree = yarip.$("blacklist-tree");
		this.tabs["blacklist"].columns = {
			blXPath: 0,
			blDeep: 1,
			blFound: 2,
			blNotFound: 3,
			blBlacklisted: 4
		}

		this.reloadObserver = new yarip.Observer(
			yarip.RELOAD_ROOT,
			yarip.VALUE,
			function() {
				var type = yarip.pageDialog.control.getValue(yarip.CHANGE, yarip.CHANGE_TYPE.none, yarip.INTEGER);
				yarip.setChangeType(type);
				yarip.pageDialog.reload();
			},
			function(type) {
				yarip.pageDialog.control.setValue(yarip.CHANGE, type, yarip.INTEGER);
				yarip.pageDialog.control.setValue(yarip.RELOAD, (new Date()).getTime(), yarip.STRING);
			}
		);

		this.listboxPages.addEventListener("select", this, false);
		this.tabs["whitelist"].tree.addEventListener("select", this, false);
		this.tabs["blacklist"].tree.addEventListener("select", this, false);
		yarip.setChangeType(this.control.getValue(yarip.CHANGE, yarip.CHANGE_TYPE.none, yarip.INTEGER));

		this.refreshPages(true);
		if (pageName) this.selectListboxItem(pageName);
//		if (pageName) this.selectTab(this.selectListboxItem(pageName));
	}

	this.unload = function()
	{
		this.reloadObserver.unregister();
		this.listboxPages.removeEventListener("select", this, false);
		this.tabs["whitelist"].tree.removeEventListener("select", this, false);
		this.tabs["blacklist"].tree.removeEventListener("select", this, false);
		this.save();
	}
}

yarip.TreeView = function(table, columns, rowcount)
{
	this.table = table ? table : null;
	this.columns = columns ? columns : null;
	this.rowCount = rowcount ? rowcount : 0;
	this.cycleHeader = function(column) {}
	this.getCellProperties = function(row, col, props) {}
	this.getCellText = function(row, col) { return this.table[row][this.columns[col.id]]; }
	this.getColumnProperties = function(colid, col, props) {}
	this.getImageSrc = function(row, col) { return null; }
	this.getLevel = function(row) { return 0; }
	this.getRowProperties = function(row, props) {}
	this.isContainer = function(row) { return false; }
	this.isSeparator = function(row) { return false; }
	this.isSorted = function(row) { return false; }
	this.setTree = function(treebox) { this.treebox = treebox; }
}
